#!/usr/bin/python

"""
game.py - main in-game scene classes
"""

import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
#from OpenGL.GLUT import *

from gamedirector3d import *
from math import *
import numpy as np

import resources
import playerbot
import world
import npc

class HUD(object):
	def __init__(self, window_size, parent):
		self.window_size = window_size
		self.parent = parent
		self.tasks = []
		self.delaytask = None
		self.delaytaskto = 0
		
		self.tutorialtask = None
		
		self.hack_progress = [0.0,0.0]
	
	def render_hack_progress(self,to1,to2):
		self.hack_progress[0] = to1
		self.hack_progress[1] = to2
	
	def drawtutorialmessages(self):
		if not self.tutorialtask == None:
			x = self.window_size[0]/2 - resources.HUD_surfs['tutorial'][self.tutorialtask].w/2
			y = self.window_size[1]/2 - 170
			resources.HUD_surfs['tutorial'][self.tutorialtask].render(x,y,1.0,0.0)
			
	def draw_tasks(self):
		for task in self.tasks:
			obj, x, y, s, d = task
			obj.render(x,y,s,d)
		self.tasks = []
		if self.parent.level_id in ['tutorial1','tutorial2','tutorial3']:
			self.drawtutorialmessages()
		if self.parent.level_id == 'boss' and self.parent.countin <= 0:
			x = self.window_size[0]/2 - resources.HUD_surfs['juggernautbig'].w/2
			y = self.window_size[1]/2 - 200
			resources.HUD_surfs['juggernautbig'].render(x,y,1.0,0.0)
			x1 = self.window_size[0]/2 - 60
			x2 = self.window_size[0]/2 - 60 + 120.0*self.parent.juggernaut.health/100
			y = self.window_size[1]/2 - 225
			glBegin(GL_QUADS)
			glColor(1, 1, 0, 0.5)
			glVertex2f(x1, y)
			glVertex2f(x2, y)
			glVertex2f(x2, y+16)
			glVertex2f(x1, y+16)
			glEnd()
		if self.delaytaskto > 0:
			if self.delaytask == 'begin':
				num = self.parent.datavaultleft
				x = self.window_size[0]/2 - resources.HUD_surfs['datavaulthackbegin'][num].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['datavaulthackbegin'][num].render(x,y,1.0,0.0)
			elif self.delaytask == 'shield':
				x = self.window_size[0]/2 - resources.HUD_surfs['shielddisable'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['shielddisable'].render(x,y,1.0,0.0)
			elif self.delaytask == 'shielddamaged':
				x = self.window_size[0]/2 - resources.HUD_surfs['shielddamaged'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['shielddamaged'].render(x,y,1.0,0.0)
			elif self.delaytask == 'datavault':
				x = self.window_size[0]/2 - resources.HUD_surfs['datavaulthack'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['datavaulthack'].render(x,y,1.0,0.0)
				surf2 = resources.HUD_surfs['datavaulthackleft'][self.parent.datavaultleft]
				x = self.window_size[0]/2 - surf2.w/2
				y = self.window_size[1]/2 + 75
				surf2.render(x,y,1.0,0.0)
			elif self.delaytask == 'sentry_destroyed':
				x = self.window_size[0]/2 - resources.HUD_surfs['sentry_destroyed'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['sentry_destroyed'].render(x,y,1.0,0.0)
			elif self.delaytask == 'sentry_disabled':
				x = self.window_size[0]/2 - resources.HUD_surfs['sentry_disabled'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['sentry_disabled'].render(x,y,1.0,0.0)
			elif self.delaytask == 'turret_disabled':
				x = self.window_size[0]/2 - resources.HUD_surfs['turret_disabled'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['turret_disabled'].render(x,y,1.0,0.0)
			elif self.delaytask == 'missile_destroyed' and not self.parent.missioncomplete:
				x = self.window_size[0]/2 - resources.HUD_surfs['missile_destroyed'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['missile_destroyed'].render(x,y,1.0,0.0)
			elif self.delaytask == 'detectedmissile' and self.parent.playeralive:
				x = self.window_size[0]/2 - resources.HUD_surfs['detectedmissile'].w/2
				y = self.window_size[1]/2 + 100
				resources.HUD_surfs['detectedmissile'].render(x,y,1.0,0.0)
			self.delaytaskto -= 1
		if self.hack_progress[0] > 0:
			x1 = self.window_size[0]/2 - 30
			x2 = self.window_size[0]/2 - 30 + 60.0*self.hack_progress[0]/self.hack_progress[1]
			y = self.window_size[1]/2 + 32
			glBegin(GL_QUADS)
			glColor(0, 1, 0, 0.5)
			glVertex2f(x1, y)
			glVertex2f(x2, y)
			glVertex2f(x2, y+16)
			glVertex2f(x1, y+16)
			glEnd()
		self.hack_progress[0] = 0.0
		if self.parent.hackerbot.hacking_state == 1:
			x = self.window_size[0]/2 - resources.HUD_surfs['targethacked'].w/2
			y = self.window_size[1]/2 + 200
			resources.HUD_surfs['targethacked'].render(x,y,1.0,0.0)
			x = self.window_size[0]/2 - resources.HUD_surfs['control_release'].w/2
			y = self.window_size[1]/2 + 160
			resources.HUD_surfs['control_release'].render(x,y,1.0,0.0)
			x = self.window_size[0]/2 - resources.HUD_surfs['control_disable'].w/2
			y = self.window_size[1]/2 + 130
			resources.HUD_surfs['control_disable'].render(x,y,1.0,0.0)
			x = self.window_size[0]/2 - resources.HUD_surfs['control_take'].w/2
			y = self.window_size[1]/2 + 100
			resources.HUD_surfs['control_take'].render(x,y,1.0,0.0)
		elif self.parent.hackerbot.hacking_state == 2:
			x = self.window_size[0]/2 - resources.HUD_surfs['targethacked'].w/2
			y = self.window_size[1]/2 + 200
			resources.HUD_surfs['targethacked'].render(x,y,1.0,0.0)
			if self.parent.hackerbot.target.type == 'sentry':
				ind = int(self.parent.hackerbot.control_to/30)
				x = self.window_size[0]/2 - resources.HUD_surfs['revive'][ind].w/2
				y = self.window_size[1]/2 + 160
				resources.HUD_surfs['revive'][ind].render(x,y,1.0,0.0)
			x = self.window_size[0]/2 - resources.HUD_surfs['control_release'].w/2
			y = self.window_size[1]/2 - 130
			resources.HUD_surfs['control_release'].render(x,y,1.0,0.0)
			if self.parent.hackerbot.target.type == 'sentry':
				x = self.window_size[0]/2 - resources.HUD_surfs['control_fire'].w/2
				y = self.window_size[1]/2 - 160
				resources.HUD_surfs['control_fire'].render(x,y,1.0,0.0)
			(x, y) = (self.window_size[0]/2 - 16, self.window_size[1]/2 - 16)
			resources.hud_target_sprites.render(x,y,1.0,2)
		if self.parent.playeralive == False:
			x = self.window_size[0]/2 - resources.HUD_surfs['missionfailed'].w/2
			y = self.window_size[1]/2 + 100
			resources.HUD_surfs['missionfailed'].render(x,y,1.0,0.0)
			x = self.window_size[0]/2 - resources.HUD_surfs[self.parent.reasonfail].w/2
			y = self.window_size[1]/2 + 0
			resources.HUD_surfs[self.parent.reasonfail].render(x,y,1.0,0.0)
		elif self.parent.missioncomplete == True:
			x = self.window_size[0]/2 - resources.HUD_surfs['missioncomplete'].w/2
			y = self.window_size[1]/2 + 100
			resources.HUD_surfs['missioncomplete'].render(x,y,1.0,0.0)
		if not self.parent.level_id == 'boss':
			if self.parent.activity_type == 'activity':
				x = self.window_size[0]/2 - resources.HUD_surfs['activity'].w/2
				y = self.window_size[1]/2 - 200
				resources.HUD_surfs['activity'].render(x,y,1.0,0.0)
			elif self.parent.activity_type == 'detected':
				x = self.window_size[0]/2 - resources.HUD_surfs['detected'].w/2
				y = self.window_size[1]/2 - 200
				resources.HUD_surfs['detected'].render(x,y,1.0,0.0)

class Console(object):
	def __init__(self, window_size):
		self.window_size = window_size
		self.text_id = None
		self.pos = 0.0
	
	def new(self, id):
		self.text_id = id
		self.pos = -self.window_size[1]+128
	
	def render(self):
		if not self.text_id == None:
			sprite = resources.console_text_sprites[self.text_id]
			sprite.render(0.0,self.pos,0.75,0)
			#if self.pos < (sprite.size-self.window_size[1]):
			if self.pos < (220-self.window_size[1]):
				self.pos += 10.0

class ProgressData(object):
    def __init__(self):
        self.current_level = "level_"
    
    def LoadProgressData(self):
        f = open(self.savepath, "r");
        self.current_level = f.readline().split()[0]
        f.close()
    
    def SaveProgressData(self):
        f = open(self.savepath, "w");
        f.write('%s\n'%(self.current_level))
        f.close()
    
    def Reset(self):
        self.current_level = "level_"

class TutorialData(object):
	def __init__(self,parent):
		self.parent = parent
		self.task = None
		self.to = 0
	
	def Update(self):
		
		if self.parent.missioncomplete == True:
			self.parent.hud.tutorialtask = None
			return
		
		if self.parent.level_id == 'tutorial1':
			if self.task == None and self.parent.hackerbot.current_asteroid == 0:
				self.task = 0
				self.parent.hud.tutorialtask = 0
				self.to = 150
			elif self.task == 0:
				self.to -= 1
				if self.to <= 0:
					self.task = 1
					self.parent.hud.tutorialtask = 1
					self.to = 150
			elif self.task == 1:
				self.to -= 1
				if self.to <= 0:
					self.task = 2
					self.parent.hud.tutorialtask = 2
					self.to = 150
			elif self.task == 2:
				self.to -= 1
				if self.to <= 0:
					self.task = 3
					self.parent.hud.tutorialtask = 3
					self.to = 150
			elif self.task == 3:
				if self.parent.hackerbot.current_asteroid == 1:
					self.task = 4
					self.parent.hud.tutorialtask = 4
					self.to = 150
			elif self.task == 4:
				self.to -= 1
				if self.to <= 0:
					self.task = 5
					self.parent.hud.tutorialtask = 5
					self.to = 150
		elif self.parent.level_id == 'tutorial2':
			if self.parent.npcs[1].state == 'disabled':
				self.parent.hud.tutorialtask = None
			if self.parent.hackerbot.current_asteroid == 0:
				if self.parent.shields[0].active:
					self.task = 6
					self.parent.hud.tutorialtask = 6
				else:
					self.parent.hud.tutorialtask = None
					self.to = 150
		elif self.parent.level_id == 'tutorial3':
			if self.parent.hackerbot.current_asteroid == 0 and not self.parent.npcs[1].state == 'disabled':
				self.parent.hud.tutorialtask = 7
			elif self.parent.hackerbot.current_asteroid in [2,3,4] and not self.parent.npcs[3].state == 'disabled':
				self.parent.hud.tutorialtask = 8
			else:
				self.parent.hud.tutorialtask = None
				

class MainGame(GameScene):
    def __init__(self, director, window_size):
        super(MainGame, self).__init__(director)
        self.window_size = window_size
        
        self.current_music = 'none'
        
        # Initialise player data
        self.progress_data = ProgressData()
        
        # frame rate recording
        self.avgframerate = -1
        self.frsamples = 0
    
    def on_switchto(self, switchtoargs):
        
        self.juggernaut = None
        self.level_id = switchtoargs[0]
        self.paused = False
        self.fadeto = 30
        self.playeralive = True
        self.missioncomplete = False
        
        if self.level_id == 'boss':
        	self.countin = 6*30
        
        # Save progress
        self.progress_data.current_level = self.level_id
        self.progress_data.SaveProgressData()
        
        # Initialise openGL stuff
        glLightfv(GL_LIGHT0, GL_POSITION,  (100, 200, 100, 0.0))
        glLightfv(GL_LIGHT0, GL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
        glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.5, 0.5, 0.5, 1.0))
        glEnable(GL_LIGHT0)
        glEnable(GL_LIGHTING)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_DEPTH_TEST)
        glShadeModel(GL_SMOOTH)
        
        # Initialise motion controls
        pygame.mouse.set_pos([320,240])
        self.mp = pygame.mouse.get_pos()
        pygame.mouse.set_visible(False)
        self.motion = playerbot.MotionControl(self,resources.controlmap)
        
        # Initialise World
        self.skybox = world.Skybox()
        self.map_limit = resources.MissionData[self.level_id].map_limit
        
        self.asteroids = []
        for asteroid in resources.MissionData[self.level_id].asteroids:
        	self.asteroids.append(world.Asteroid(type=1,id=asteroid[0],pos=asteroid[1]))
        
        self.shields = []
        for shield in resources.MissionData[self.level_id].shields:
        	self.shields.append(world.Shield(self,pos=shield[0],radius=shield[1]))
        
        self.explosions = world.Explosions()
        self.projectiles = world.Projectiles(self)
        
        self.missile_cleanup = []
        
        self.tutorialdata = TutorialData(self)
        if self.level_id == 'tutorial2':
        	pass
        elif self.level_id == 'tutorial3':
        	pass
        
        # Initialise Player and NPCs
        #astid = resources.MissionData[self.level_id].hackerbotstart
        #startpos = self.asteroids[astid].startpoint + self.asteroids[astid].position
        (pos,vel,yaw) = resources.MissionData[self.level_id].hackerbotstart
        self.hackerbot = playerbot.Hackerbot(self,pos=pos,vel=vel,yaw=yaw)
        
        self.npcs = []
        self.datavaultleft = 0
        for npcs in resources.MissionData[self.level_id].npcs:
        	type = npcs[0]
        	if type == 'datavault':
        		self.datavaultleft += 1
        	if type == 'sentry':
        		self.npcs.append(npc.Sentry(self,np.array(npcs[1])))
        	elif type == 'juggernaut':
				self.npcs.append(npc.StaticHackable(self,np.array([0.0,0.0,0.0]),0.0,0.0,0,type))
				self.juggernaut = self.npcs[-1]
        	elif type in ['shieldgen','datavault','turret']:
				if type == 'shieldgen':
					h_obj = self.shields[npcs[3]]
				else:
					h_obj = 0
				
				if npcs[2] == 0:
					i = np.argmin(self.asteroids[npcs[1]].centers[:,0])
					corr = 0.0
				elif npcs[2] == 1:
					i = np.argmax(self.asteroids[npcs[1]].centers[:,0])
					corr = -1.0
				elif npcs[2] == 2:
					i = np.argmin(self.asteroids[npcs[1]].centers[:,1])
					corr = 0.5
				elif npcs[2] == 3:
					i = np.argmax(self.asteroids[npcs[1]].centers[:,1])
					corr = -0.5
				elif npcs[2] == 4:
					i = np.argmin(self.asteroids[npcs[1]].centers[:,2])
					corr = -0.5
				elif npcs[2] == 5:
					i = np.argmax(self.asteroids[npcs[1]].centers[:,2])
					corr = 0.5
				rel_pos = self.asteroids[npcs[1]].centers[i,:]
				dist = sqrt(np.sum((rel_pos)**2))
				position = ((dist+corr)/dist)*rel_pos+self.asteroids[npcs[1]].position
				yaw = -atan2(rel_pos[1],rel_pos[0])
				hordist = sqrt(rel_pos[0]*rel_pos[0] + rel_pos[1]*rel_pos[1])
				if hordist > 0.0:
					pitch = -atan(rel_pos[2]/hordist)
				else:
					pitch = pi/2
				self.npcs.append(npc.StaticHackable(self,position,pitch,yaw,h_obj,type))
				if type == 'shieldgen':
					self.shields[npcs[3]].generators.append(self.npcs[-1])
        
        self.activity_type = None
        self.on_alert = False
        
        # Initialise HUD
        self.hud = HUD(self.window_size, self)
        self.console = Console(self.window_size)
        
        # set handles
        self.skybox.h_motion = self.motion
        #self.spaceobject.h_motion = self.motion
        self.hackerbot.h_motion = self.motion
        self.hackerbot.h_hud = self.hud
        self.hackerbot.h_asteroids = self.asteroids
        self.motion.h_hackerbot = self.hackerbot
        self.motion.h_projectiles = self.projectiles
        self.motion.h_explosions = self.explosions
        self.explosions.h_motion = self.motion
        self.projectiles.h_motion = self.motion
        self.projectiles.h_asteroids = self.asteroids
        self.projectiles.h_hackerbot = self.hackerbot
        self.projectiles.h_npcs = self.npcs
        self.projectiles.h_shields = self.shields
        self.projectiles.h_explosions = self.explosions
        
        for asteroid in self.asteroids:
        	asteroid.h_motion = self.motion
        for shield in self.shields:
        	shield.h_hackerbot = self.hackerbot
        	shield.h_explosions = self.explosions
        for n in self.npcs:
        	n.h_hud = self.hud
        	n.h_shields = self.shields
        	n.h_asteroids = self.asteroids
        	n.h_hackerbot = self.hackerbot
        	n.h_projectiles = self.projectiles
        	
        # test explosions
        #self.explosions.New(self.npcs[1].position,0)
        
        if not self.level_id == 'boss':
			self.hud.delaytask = 'begin'
			self.hud.delaytaskto = 200
        
        # temporary code for missile spawner
    	self.missileto = 5*30
        
        # start music
        if not self.current_music == resources.MissionData[self.level_id].music:
            self.current_music = resources.MissionData[self.level_id].music
            pygame.mixer.music.stop()
            if not self.current_music == 'none':
                pygame.mixer.music.load(resources.musicpaths[self.current_music])
                pygame.mixer.music.set_volume(0.5)
                pygame.mixer.music.play(-1)

    def on_update(self):
        
        # framerate tracking
        self.frsamples += 1
        if self.frsamples == 1:
            self.avgframerate = self.director.framerate
        else:
            self.avgframerate = self.avgframerate + (self.director.framerate - self.avgframerate)/(self.frsamples)
        
        if self.paused:
			return
        
        if self.level_id == 'boss':
        	if self.countin > 0:
        		self.countin -= 1
        
        if self.level_id in ['tutorial1','tutorial2','tutorial3']:
        	self.tutorialdata.Update()
        
        # Update timeouts
        if self.fadeto > 0:
        	self.fadeto -= 1
        if self.playeralive == False and self.fadeto == 1:
        	self.director.change_scene('maingame', [self.level_id])
        elif self.missioncomplete == True and self.fadeto == 1:
        	current_ind = resources.MissionData['levels'].index(self.level_id)
        	if current_ind == (len(resources.MissionData['levels'])-1):
        		self.director.change_scene('victoryscene', [])
        	else:
        		next_level = resources.MissionData['levels'][current_ind+1]
        		if not resources.MissionData[next_level].music == self.current_music:
        			for i in xrange(30):
        				time = self.director.clock.tick(self.director.fps)
        				pygame.mixer.music.set_volume(0.5-(0.5/30.0)*i)
        		self.director.change_scene('maingame', [next_level])
        
        # Update motion control
        self.motion.UpdateMotion()
        
        # Update player
        self.hackerbot.Update()
        
        # Update NPCs
        for n in self.npcs:
        	n.Update()
        
        # check for activity flags
        self.activity_type = None
        investigate = False
        detect = False
        for n in self.npcs:
        	if n.state == 'investigate':
        		investigate = True
        	elif n.state == 'attack':
        		detect = True
        	elif n.type == 'turret':
        		if n.player_seen == True:
        			detect = True
        if detect:
        	self.activity_type = 'detected'
        elif investigate:
        	self.activity_type = 'activity'
        
        # Update environment
        for shield in self.shields:
        	shield.Update()
        self.projectiles.Update()
        self.explosions.Update()
    	
    	# Check for cleaning up spent missiles
    	for m in self.missile_cleanup:
    		self.npcs.remove(m)
    	self.missile_cleanup = []
    	
    	# temporary code for missile spawner
    	"""
    	self.missileto -= 1
    	if self.missileto <= 0:
    		self.missileto = 30*30
    		self.npcs.append(npc.Missile(self, [-200.0,20.0,0.0], [1.0,0.0,0.0]))
    		self.npcs[-1].h_hud = self.hud
        	self.npcs[-1].h_shields = self.shields
        	self.npcs[-1].h_asteroids = self.asteroids
        	self.npcs[-1].h_hackerbot = self.hackerbot
        	self.npcs[-1].h_projectiles = self.projectiles
        	self.npcs[-1].h_npcs = self.npcs
        	self.npcs[-1].h_explosions = self.explosions
        """
    	
    def ProcessPauseOptions(self, event):
    	if event.type == KEYDOWN:
			if event.key == K_q:
				self.director.change_scene('titlescene', [])
			elif event.key == K_r:
				self.director.change_scene('maingame', [self.level_id])
    
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
            	if self.paused:
            		self.paused = False
            	else:
            		self.paused = True
                #self.director.change_scene(None, [])
            if event.type == KEYDOWN or event.type == KEYUP or event.type == MOUSEBUTTONDOWN or event.type == MOUSEBUTTONUP:
            	if self.paused:
            		self.ProcessPauseOptions(event)
            	else:
                	self.motion.ProcessKeyEvent(event)
        self.mp = pygame.mouse.get_pos()
        pygame.mouse.set_pos([320,240])
    
    def render_pausescreen(self, screen):
		glBegin(GL_QUADS)
		glColor(0, 0, 0, 0.5)
		glVertex2f(0, 0)
		glVertex2f(self.window_size[0], 0)
		glVertex2f(self.window_size[0], self.window_size[1])
		glVertex2f(0, self.window_size[1])
		glEnd()
		
		x = self.window_size[0]/2 - resources.HUD_surfs['paused'].w/2
		y = self.window_size[1]/2 + 150
		resources.HUD_surfs['paused'].render(x,y,1.0,0.0)
		
		x = self.window_size[0]/2 - resources.HUD_surfs['resume'].w/2
		y = self.window_size[1]/2 + 100
		resources.HUD_surfs['resume'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['quit'].w/2
		y = self.window_size[1]/2 + 60
		resources.HUD_surfs['quit'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['reset'].w/2
		y = self.window_size[1]/2 + 20
		resources.HUD_surfs['reset'].render(x,y,1.0,0.0)
		
		surf2 = resources.HUD_surfs['datavaulthackleft_red'][self.datavaultleft]
		x = self.window_size[0]/2 - surf2.w/2
		y = self.window_size[1]/2 -16
		surf2.render(x,y,1.0,0.0)
		
		x = self.window_size[0]/2 - resources.HUD_surfs['controls'].w/2
		y = self.window_size[1]/2 - 60
		resources.HUD_surfs['controls'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['cont001'].w/2
		y = self.window_size[1]/2 - 90
		resources.HUD_surfs['cont001'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['cont002'].w/2
		y = self.window_size[1]/2 - 120
		resources.HUD_surfs['cont002'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['cont003'].w/2
		y = self.window_size[1]/2 - 150
		resources.HUD_surfs['cont003'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['cont004'].w/2
		y = self.window_size[1]/2 - 180
		resources.HUD_surfs['cont004'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.HUD_surfs['cont005'].w/2
		y = self.window_size[1]/2 - 210
		resources.HUD_surfs['cont005'].render(x,y,1.0,0.0)
    
    def render_fade(self, screen):
		if self.playeralive == True and self.missioncomplete == False:
			fade = self.fadeto/30.0
		else:
			fade = max(0.0,1.0 - self.fadeto/30.0)
		glBegin(GL_QUADS)
		glColor(0, 0, 0, fade)
		glVertex2f(0, 0)
		glVertex2f(self.window_size[0], 0)
		glVertex2f(self.window_size[0], self.window_size[1])
		glVertex2f(0, self.window_size[1])
		glEnd()
    
    def draw(self, screen):
    	
    	# Initialise view
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(60.0, self.window_size[0]/float(self.window_size[1]), 1, 1000.0)
        
        # setup camera perspective on the world
        self.motion.set_view()
        
        # clear screen
        glMatrixMode(GL_MODELVIEW)
        glClearColor(*(0.0,0.0,0.0,0.0))
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        # render skybox
        glDisable(GL_LIGHTING)
        glDisable(GL_DEPTH_TEST)
        glDepthMask(GL_FALSE)
        self.skybox.render()
        glEnable(GL_LIGHTING)
        glEnable(GL_DEPTH_TEST)
        glDepthMask(GL_TRUE)
        
        # render world objects
        #self.spaceobject.render()
        self.hackerbot.render()
        for npc in self.npcs:
        	npc.render()
        for shield in self.shields:
        	shield.render()
        self.projectiles.render()
        for asteroid in self.asteroids:
        	if asteroid == self.asteroids[self.hackerbot.current_asteroid]:
        		continue
        	asteroid.render()
        self.asteroids[self.hackerbot.current_asteroid].render() # render current asteroid last (for blending)
        self.explosions.render()
        
        # debug render
        """
        ast = self.asteroids[self.hackerbot.current_asteroid]
        pos = ast.position + ast.centers[self.hackerbot.cl_ind]
        glLoadIdentity()
        glPushMatrix()
        glTranslate(*(pos[1],-pos[2],-pos[0]))
        glScale(1.0, 1.0, 1.0)
        glCallList(resources.model_hackerbot.obj.gl_list)
        glPopMatrix()
        """
        
        # draw HUD
        
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glViewport(0, 0, self.window_size[0], self.window_size[1])
        glOrtho(0, self.window_size[0], 0, self.window_size[1], -50, 50)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        glDisable(GL_LIGHTING)
        glDisable(GL_DEPTH_TEST)
        
        glPushMatrix()
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        self.hud.draw_tasks()
        if self.hackerbot.hacking_state == 0:
        	self.console.text_id = None
        self.console.render()
        if self.fadeto > 0:
        	self.render_fade(screen)
        if self.paused:
        	self.render_pausescreen(screen)
        glDisable(GL_BLEND)
        glPopMatrix()
    
    def on_draw(self, screen):
        self.draw(screen)


