#!/usr/bin/python

"""
npc.py - classes for NPCs
"""

import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
#from OpenGL.GLUT import *

from gamedirector3d import *
from math import *
import numpy as np

import resources

class StaticHackable(object):
	def __init__(self,parent,position,pitch,yaw,h_obj,type):
		self.type = type
		self.parent = parent
		if self.type == 'shieldgen':
			self.hack_to = 45
		if self.type == 'turret':
			self.hack_to = 45
		elif self.type == 'datavault':
			self.hack_to = 90
		elif self.type == 'juggernaut':
			self.hack_to = 30*30
		self.state = 'active'
		self.position = position
		self.h_obj = h_obj
		self.pitch = pitch
		self.yaw = yaw
		self.radius = 3.0
		self.viewdistlong = 150
		self.viewdistlongrogue = 400.0
		self.player_seen = False
		self.fireto = 40
		self.max_hack_range = 70.0
		
		if type == 'juggernaut':
			self.radius = 20.0
		self.health = 100
		self.disto = 0
		self.missileto = 10*30
	
	def check_los(self, target_pos):
		d2 = sqrt(np.sum((target_pos-self.position)**2))
		for asteroid in self.h_asteroids:
			d1 = sqrt(np.sum((np.cross(asteroid.position-self.position,asteroid.position-target_pos))**2))
			dist = d1/d2
			t = -np.dot((self.position-asteroid.position),target_pos-self.position)/pow(d2,2)
			if t >= 0.0 and t <= 1.0 and dist < asteroid.radius:
				return False
		return True
	
	def LookforRogue(self):
		if self.parent.playeralive == False:
			return False
		relpos = self.h_hackerbot.target.position-self.position
		dist2player = sqrt(np.sum(relpos**2))
		los = self.check_los(self.h_hackerbot.target.position)
		if dist2player < self.viewdistlongrogue and los == True:
			return True
		else:
			return False
	
	def LookforPlayer(self):
		if self.parent.playeralive == False:
			return False
		relpos = self.h_hackerbot.position-self.position
		dist2player = sqrt(np.sum(relpos**2))
		los = self.check_los(self.h_hackerbot.position)
		if dist2player < self.viewdistlong and los == True:
			return True
		else:
			return False
	
	def Update(self):
		if self.state == 'disabled':
			pass
		elif self.state == 'hacked':
			pass
		elif self.state == 'active' and self.type in ['turret','juggernaut']:
			
			# Check for rogue
			if self.parent.on_alert == True:
				found2 = self.LookforRogue()
				if found2 == True:
					self.fireto -= 1
					if self.fireto == 0:
						relpos = self.h_hackerbot.target.position-self.position
						dist = sqrt(np.sum(relpos**2))
						if dist > 0:
							self.h_projectiles.New(self.position,10*relpos/dist,0,self)
						self.fireto = 40
					return
			
			found = self.LookforPlayer()
			if found == True:
				if self.player_seen == False:
					resources.soundfx['check'].stop()
					resources.soundfx['alert'].play()
					self.player_seen = True
				self.fireto -= 1
				if self.fireto == 0:
					relpos = self.h_hackerbot.position-self.position
					dist = sqrt(np.sum(relpos**2))
					if dist > 0:
						self.h_projectiles.New(self.position,10*relpos/dist,0,self)
					self.fireto = 40
			else:
				self.player_seen = False
				self.fireto = 40
				
			if self.type == 'juggernaut':
				self.missileto -= 1
				if self.missileto <= 0 and self.h_hackerbot.hacking_state == 0:
					self.missileto = 10*30
					if found or self.h_hackerbot.current_asteroid == -1:
						relpos = self.h_hackerbot.position-self.position
						dist = sqrt(np.sum(relpos**2))
						if dist > 0.0:
							vel = 1.5*relpos/dist
						else:
							vel = [1.5,0,0]
					else:
						relpos = self.h_asteroids[self.h_hackerbot.current_asteroid].position + np.array([0.0,0.0,35.0]) - self.position
						dist = sqrt(np.sum(relpos**2))
						if dist > 0.0:
							vel = 1.5*relpos/dist
						else:
							vel = [1.5,0,0]
					resources.soundfx['missile'].play()
					self.parent.hud.delaytask = 'detectedmissile'
					self.parent.hud.delaytaskto = 200
					self.parent.npcs.append(Missile(self.parent, self.position, vel))
					self.parent.npcs[-1].h_hud = self.parent.hud
					self.parent.npcs[-1].h_shields = self.parent.shields
					self.parent.npcs[-1].h_asteroids = self.parent.asteroids
					self.parent.npcs[-1].h_hackerbot = self.parent.hackerbot
					self.parent.npcs[-1].h_projectiles = self.parent.projectiles
					self.parent.npcs[-1].h_npcs = self.parent.npcs
					self.parent.npcs[-1].h_explosions = self.parent.explosions
	
	def hit(self,type):
		if self.type == 'shieldgen' and not self.state == 'disabled':
			self.state = 'disabled'
			self.h_obj.generators.remove(self)
			if len(self.h_obj.generators) == 0:
				self.h_obj.active = False
				self.h_hud.delaytask = 'shield'
				self.h_hud.delaytaskto = 150
			else:
				self.h_hud.delaytask = 'shielddamaged'
				self.h_hud.delaytaskto = 150
			if self.parent.level_id == 'boss':
				self.parent.juggernaut.missileto = 10*30
		elif self.type == 'juggernaut':
			self.health -= 30
			self.missileto = 10*30
			if self.health <= 0:
				self.health = 0
				self.state = 'disabled'
				self.parent.explosions.New(self.position,1,scale=5.0)
				self.parent.fadeto = 150
				self.parent.missioncomplete = True
				self.parent.motion.viewtype = 0
	
	def render(self):
		glLoadIdentity()
		glPushMatrix()
		
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(self.yaw*180/pi, 0, 1, 0)
		glRotate(self.pitch*180/pi, 1, 0, 0)
		
		glScale(1.0, 1.0, 1.0)
		if self.type == 'shieldgen':
			glCallList(resources.model_shieldgen.obj.gl_list)
		elif self.type == 'datavault':
			glCallList(resources.model_datavault.obj.gl_list)
		elif self.type == 'turret':
			glCallList(resources.model_turret.obj.gl_list)
		elif self.type == 'juggernaut':
			if not self.state == 'disabled':
				glCallList(resources.model_juggernaut.obj.gl_list)
		glPopMatrix()

class Sentry(object):
	def __init__(self,parent,waypoints):
		self.type = 'sentry'
		self.parent = parent
		self.waypoints = waypoints
		
		self.viewdistlong = 150.0
		self.viewdistlongrogue = 400.0
		self.viewdistshort = 100.0
		
		self.hack_to = 45
        
		self.state = 'patrol'
		self.position = self.waypoints[0,:]
		self.pitch = 0.0
		self.yaw = 0.0
        
		self.currentwp = 1
		self.radius = 4.5
		
		self.attentionto = 30
		self.fireto = 0
		self.disableto = 0
		
		self.max_hack_range = 50.0
	
	def check_los(self, target_pos):
		d2 = sqrt(np.sum((target_pos-self.position)**2))
		if d2 == 0.0:
			return False
		for asteroid in self.h_asteroids:
			d1 = sqrt(np.sum((np.cross(asteroid.position-self.position,asteroid.position-target_pos))**2))
			dist = d1/d2
			t = -np.dot((self.position-asteroid.position),target_pos-self.position)/pow(d2,2)
			if t >= 0.0 and t <= 1.0 and dist < asteroid.radius:
				return False
		return True
	
	def LookforRogue(self):
		if self.parent.playeralive == False:
			return False
		relpos = self.h_hackerbot.target.position-self.position
		dist2player = sqrt(np.sum(relpos**2))
		los = self.check_los(self.h_hackerbot.target.position)
		if dist2player < self.viewdistlongrogue and los == True:
			DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
			dpx = DCM[0][0]*relpos[0] + DCM[0][1]*relpos[1] + DCM[0][2]*relpos[2]
			dpy = DCM[1][0]*relpos[0] + DCM[1][1]*relpos[1] + DCM[1][2]*relpos[2]
			dpz = DCM[2][0]*relpos[0] + DCM[2][1]*relpos[1] + DCM[2][2]*relpos[2]
			
			if dist2player <= self.viewdistlongrogue:
				viewang = 90
			else:
				viewang = 60
			
			if dpx == 0.0:
				ang1 = 90
				ang2 = 90
			else:
				ang1 = (180/pi)*atan(fabs(dpy/dpx))
				ang2 = (180/pi)*atan(fabs(dpz/dpx))
			if dpx > 0 and ang1 < viewang and ang2 < viewang:
				if dist2player < self.viewdistshort:
					return 'short'
				else:
					return 'long'
			else:
				return False
		else:
			return False
	
	def LookforPlayer(self):
		if self.parent.playeralive == False:
			return False
		relpos = self.h_hackerbot.position-self.position
		dist2player = sqrt(np.sum(relpos**2))
		los = self.check_los(self.h_hackerbot.position)
		if dist2player < self.viewdistlong and los == True:
			DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
			dpx = DCM[0][0]*relpos[0] + DCM[0][1]*relpos[1] + DCM[0][2]*relpos[2]
			dpy = DCM[1][0]*relpos[0] + DCM[1][1]*relpos[1] + DCM[1][2]*relpos[2]
			dpz = DCM[2][0]*relpos[0] + DCM[2][1]*relpos[1] + DCM[2][2]*relpos[2]
			
			if dist2player <= self.viewdistshort:
				viewang = 90
			else:
				viewang = 60
			
			if dpx == 0.0:
				ang1 = 90
				ang2 = 90
			else:
				ang1 = (180/pi)*atan(fabs(dpy/dpx))
				ang2 = (180/pi)*atan(fabs(dpz/dpx))
			if dpx > 0 and ang1 < viewang and ang2 < viewang:
				if dist2player < self.viewdistshort:
					return 'short'
				else:
					return 'long'
			else:
				return False
		else:
			return False
	
	def MovetoWaypoint(self,pos,speed='slow'):
		rel_pos = pos-self.position
		DCM2 = np.array([[cos(self.yaw),-sin(self.yaw),0.0],\
			[sin(self.yaw),cos(self.yaw),0.0],[0.0,0.0,1.0]])
		vertex_c = [np.dot(rel_pos,DCM2[0]),np.dot(rel_pos,DCM2[1]),np.dot(rel_pos,DCM2[2])]
		hordist = sqrt(vertex_c[0]*vertex_c[0] + vertex_c[1]*vertex_c[1])
		if hordist > 0.0:
			azimuth = atan2(vertex_c[1],vertex_c[0])
			elev = -atan(vertex_c[2]/hordist)
		else:
			azimuth = 0.0
			elev = 0.0
		if speed == 'slow':
			turn_rate = 0.06
		elif speed == 'fast':
			turn_rate = 0.12
		if self.pitch < elev:
			self.pitch += min(turn_rate,(elev-self.pitch))
		elif self.pitch > elev:
			self.pitch -= min(turn_rate,(self.pitch-elev))
		if self.pitch >= pi/2:
			self.pitch = pi/2-0.0001
		elif self.pitch <= -pi/2:
			self.pitch = -pi/2+0.0001
		if azimuth > 0:
			self.yaw -= min(turn_rate,fabs(azimuth))
		elif azimuth < 0:
			self.yaw += min(turn_rate,fabs(azimuth))
		
		dist2target = sqrt(np.sum((pos-self.position)**2))
		if fabs(self.pitch-elev) < 0.2 and fabs(azimuth) < 0.2 and dist2target > 0.0:
			relpos2 = 0.5*(rel_pos/dist2target)
			self.position = self.position + relpos2
		
		self.CorrectTrajforObstacles()
	
	def CorrectTrajforObstacles(self,control='sentry'):
		
		dists = []
		for asteroid in self.h_asteroids:
			dists.append(sqrt(np.sum((self.position-asteroid.position)**2)))
		ind = dists.index(min(dists))
		if min(dists) < (self.h_asteroids[ind].radius+2.0):
			self.position = (self.h_asteroids[ind].radius+2.0)*(self.position-self.h_asteroids[ind].position)/min(dists) + self.h_asteroids[ind].position
		
		if control == 'player':
			dists = []
			for shield in self.h_shields:
				if shield.active:
					dists.append(sqrt(np.sum((self.position-shield.position)**2)))
				else:
					dists.append(10000.0)
			if len(dists) > 0:
				ind = dists.index(min(dists))
				if min(dists) < (self.h_shields[ind].radius+2.0):
					self.position = (self.h_shields[ind].radius+2.0)*(self.position-self.h_shields[ind].position)/min(dists) + self.h_shields[ind].position
	
	def SetInvestigationWaypoints(self):
		if self.h_hackerbot.current_asteroid == -1:
			dists = []
			for asteroid in self.h_asteroids:
				dists.append(sqrt(np.sum((self.h_hackerbot.position-asteroid.position)**2)))
			asteroid_target = self.h_asteroids[dists.index(min(dists))]
		else:
			asteroid_target = self.h_asteroids[self.h_hackerbot.current_asteroid]
		
		coords = [(1,-1),(1,1),(-1,1),(-1,-1)]
		dists = []
		waypoints = []
		for i in xrange(4):
			dp = 2.0*asteroid_target.radius*np.array([coords[i][0],coords[i][1],0.0])
			wp = asteroid_target.position + dp
			dists.append(sqrt(np.sum((wp-self.position)**2)))
			waypoints.append(wp)
		ind = dists.index(min(dists))
		if ind == 0:
			self.waypoints_in = np.array([waypoints[0],waypoints[1],waypoints[2],waypoints[3]])
		elif ind == 1:
			self.waypoints_in = np.array([waypoints[1],waypoints[2],waypoints[3],waypoints[0]])
		elif ind == 2:
			self.waypoints_in = np.array([waypoints[2],waypoints[3],waypoints[0],waypoints[1]])
		elif ind == 3:
			self.waypoints_in = np.array([waypoints[3],waypoints[0],waypoints[1],waypoints[2]])
		
		self.waypoints_in = np.array(self.waypoints_in)
		self.currentwp_in = 0
	
	def Update(self):
		
		if self.state == 'dead':
			pass
		elif self.state == 'disabled':
			self.disableto -= 1
			if self.disableto <= 0:
				self.state = 'patrol'
		elif self.state == 'hacked':
			pass
		elif self.state == 'patrol':
			
			#print "patrol: attention %d"%(self.attentionto)
			self.fireto = 30
			# move to waypoint
			dist2target = sqrt(np.sum((self.waypoints[self.currentwp,:]-self.position)**2))
			if dist2target < 1.0:
				self.currentwp = (self.currentwp+1) % self.waypoints.shape[0]
			self.MovetoWaypoint(self.waypoints[self.currentwp,:])
			
			# Check for rogue
			if self.parent.on_alert == True:
				found2 = self.LookforRogue()
				if found2 in ['short','long']:
					self.state = 'attack'
					return
			
			# Check for player
			found = self.LookforPlayer()
			if found == 'short':
				self.attentionto -= 2
			elif found == 'long':
				self.attentionto -= 1
			else:
				self.attentionto = min(self.attentionto + 1,30)
			
			# look for state change based on noticed player
			if self.attentionto < 0 and found == 'short':
				#print "switching to attack!"
				resources.soundfx['check'].stop()
				resources.soundfx['alert'].play()
				self.state = 'attack'
			elif self.attentionto < 0 and found == 'long':
				#print "setting investigation waypoints"
				resources.soundfx['check'].play()
				self.state = 'investigate'
				self.SetInvestigationWaypoints()
		
		elif self.state == 'investigate':
			
			#print "investigating: wp %d "%(self.currentwp_in), self.waypoints_in[self.currentwp_in,:]
			self.fireto = 15
			# move to waypoint
			dist2target = sqrt(np.sum((self.waypoints_in[self.currentwp_in,:]-self.position)**2))
			if dist2target < 1.0:
				self.currentwp_in += 1
				if self.currentwp_in == 4:
					self.state = 'patrol'
					self.attentionto = 30
					#print "giving up"
					return
			self.MovetoWaypoint(self.waypoints_in[self.currentwp_in,:])
			
			# Check for player, change state
			found = self.LookforPlayer()
			if found == 'short' or found == 'long':
				#print "switching to attack!"
				resources.soundfx['check'].stop()
				resources.soundfx['alert'].play()
				self.state = 'attack'
			
		elif self.state == 'attack':
			
			if self.fireto > 0:
				self.fireto -= 1
			
			# hunting rogue
			if self.parent.on_alert == True:
				self.MovetoWaypoint(self.h_hackerbot.target.position,'fast')
				found = self.LookforRogue()
				if found in ['short','long'] and self.fireto == 0:
					relpos = self.h_hackerbot.target.position-self.position
					dist = sqrt(np.sum(relpos**2))
					if dist > 0:
						self.h_projectiles.New(self.position,10*relpos/dist,0,self)
					self.fireto = 60
				return
			
			#print "attack"
			self.MovetoWaypoint(self.h_hackerbot.position,'fast')
			
			found = self.LookforPlayer()
			if found == False:
				self.state = 'investigate'
				#print "setting investigation waypoints"
				self.SetInvestigationWaypoints()
			elif found == 'short' and self.fireto == 0:
				#print "fire!"
				relpos = self.h_hackerbot.position-self.position
				dist = sqrt(np.sum(relpos**2))
				if dist > 0:
					self.h_projectiles.New(self.position,10*relpos/dist,0,self)
				self.fireto = 60
	
	def hit(self,type):
		self.state = 'dead'
		self.h_hud.delaytask = 'sentry_destroyed'
		self.h_hud.delaytaskto = 150
		if self.h_hackerbot.hacking_state == 2 and self.h_hackerbot.target == self:
			self.h_hackerbot.hacking_state = 0
			self.h_hackerbot.h_motion.mbdown = 0
			self.h_hackerbot.hack_to = 0
			self.h_hackerbot.h_motion.viewtype = 0
	
	def render(self):
		if self.h_hackerbot.hacking_state == 2 and self.h_hackerbot.target == self:
			return
		if not self.state == 'dead':
			glLoadIdentity()
			glPushMatrix()
		
			glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
			glRotate(self.yaw*180/pi, 0, 1, 0)
			glRotate(self.pitch*180/pi, 1, 0, 0)
		
			glScale(1.0, 1.0, 1.0)
			glCallList(resources.model_sentry.obj.gl_list)
		
			glPopMatrix()

class Missile(object):
	def __init__(self,parent,pos,vel):
		self.type = 'missile'
		self.parent = parent
		
		self.viewdistlong = 200.0
		
		self.hack_to = 10
        
		self.tracking_to = 30
        
		self.state = 'patrol'
		self.position = np.array(pos)
		self.velocity = np.array(vel)
		
		hordist = sqrt(self.velocity[0]*self.velocity[0] + self.velocity[1]*self.velocity[1])
		if hordist > 0.0:
			self.yaw = atan2(self.velocity[1],self.velocity[0])
			self.pitch = -atan(self.velocity[2]/hordist)
		else:
			self.yaw = 0.0
			if self.velocity[2] > 0.0:
				self.pitch = -pi/2
			else:
				self.pitch = pi/2
		
		#self.pitch = 0.0
		#self.yaw = 0.0
        
		self.radius = 2.5
		
		self.max_hack_range = 100.0
		
		self.shield_safety = 60
	
	def check_los(self, target_pos):
		d2 = sqrt(np.sum((target_pos-self.position)**2))
		for asteroid in self.h_asteroids:
			d1 = sqrt(np.sum((np.cross(asteroid.position-self.position,asteroid.position-target_pos))**2))
			dist = d1/d2
			t = -np.dot((self.position-asteroid.position),target_pos-self.position)/pow(d2,2)
			if t >= 0.0 and t <= 1.0 and dist < asteroid.radius:
				return False
		return True
	
	def LookforPlayer(self):
		if self.parent.playeralive == False:
			return False
		relpos = self.h_hackerbot.position-self.position
		dist2player = sqrt(np.sum(relpos**2))
		los = self.check_los(self.h_hackerbot.position)
		if dist2player < self.viewdistlong and los == True:
			return 'short'
		else:
			return False
	
	def MovetoWaypoint(self,pos):
		rel_pos = pos-self.position
		DCM2 = np.array([[cos(self.yaw),-sin(self.yaw),0.0],\
			[sin(self.yaw),cos(self.yaw),0.0],[0.0,0.0,1.0]])
		dpx = DCM2[0][0]*rel_pos[0] + DCM2[1][0]*rel_pos[1] + DCM2[2][0]*rel_pos[2]
		dpy = DCM2[0][1]*rel_pos[0] + DCM2[1][1]*rel_pos[1] + DCM2[2][1]*rel_pos[2]
		dpz = DCM2[0][2]*rel_pos[0] + DCM2[1][2]*rel_pos[1] + DCM2[2][2]*rel_pos[2]
		vertex_c = np.array([dpx,dpy,dpz])
		hordist = sqrt(vertex_c[0]*vertex_c[0] + vertex_c[1]*vertex_c[1])
		if hordist > 0.0:
			azimuth = atan2(vertex_c[1],vertex_c[0])
			elev = -atan(vertex_c[2]/hordist)
		else:
			azimuth = 0.0
			elev = 0.0
		turn_rate = 0.12
		if azimuth > -pi/2 and azimuth < pi/2:
			if self.pitch < elev:
				self.pitch += min(turn_rate,(elev-self.pitch))
			elif self.pitch > elev:
				self.pitch -= min(turn_rate,(self.pitch-elev))
			if self.pitch >= pi/2:
				self.pitch = pi/2-0.0001
			elif self.pitch <= -pi/2:
				self.pitch = -pi/2+0.0001
		if azimuth > 0:
			self.yaw += min(turn_rate,fabs(azimuth))
		elif azimuth < 0:
			self.yaw -= min(turn_rate,fabs(azimuth))
		
		DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
		speed = 1.5
		self.velocity[0] = DCM[0][0]*speed
		self.velocity[1] = DCM[1][0]*speed
		self.velocity[2] = DCM[2][0]*speed
		self.position[0] += self.velocity[0]
		self.position[1] += self.velocity[1]
		self.position[2] += self.velocity[2]
	
	def getprojectile_closestapproach(self, target_pos):
		posprev = self.position-self.velocity
		d2 = sqrt(np.sum((self.velocity)**2))
		d1 = sqrt(np.sum((np.cross(target_pos-posprev,target_pos-self.position))**2))
		dist = d1/d2
		t = -np.dot((posprev-target_pos),self.velocity)/pow(d2,2)
		if t < 0:
			dist = sqrt(np.sum((target_pos-posprev)**2))
		elif t > 1.0:
			dist = sqrt(np.sum((target_pos-self.position)**2))
		return dist
	
	def CheckDetonate(self):
		if sqrt(np.sum((self.position)**2)) > self.parent.map_limit:
			self.h_explosions.New(self.position,1,scale=2.0)
			self.hit(0)
			return
		if self.parent.playeralive: # check for player hit
			dist = self.getprojectile_closestapproach(self.h_hackerbot.position)
			if dist <= 5.0 and self.parent.playeralive and not self.parent.missioncomplete:
				self.parent.playeralive = False
				self.parent.reasonfail = 'death_destroy'
				self.parent.fadeto = 90
				self.parent.motion.mbdown = 0
				self.parent.motion.viewtype = 0
				self.h_hackerbot.hacking_state = 0
				self.h_explosions.New(self.h_hackerbot.position,1,scale=2.0)
				self.hit(0)
				return
		for shield in self.h_shields:
			dist = self.getprojectile_closestapproach(shield.position)
			if dist <= shield.radius and shield.active and self.shield_safety <= 0:
				self.h_explosions.New(self.position-self.velocity,0,scale=2.0)
				self.hit(0)
				return
		for npc in self.h_npcs:
			if not npc.state == 'dead' and not npc == self and self.shield_safety <= 0:
				dist = self.getprojectile_closestapproach(npc.position)
				if dist <= npc.radius:
					self.h_explosions.New(self.position,0,scale=3.0)
					self.hit(0)
					npc.hit(0)
					return
		for asteroid in self.h_asteroids:
			dist = self.getprojectile_closestapproach(asteroid.position)
			if dist <= asteroid.radius:
				self.h_explosions.New(self.position-self.velocity,0,scale=2.0)
				self.hit(0)
				return
	
	def Update(self):
		
		if self.state == 'dead':
			self.parent.missile_cleanup.append(self)
		elif self.state == 'hacked':
			pass
		elif self.state == 'patrol':
			
			self.shield_safety = max(0,self.shield_safety-1)
			
			# fly in straight line
			self.position[0] += self.velocity[0]
			self.position[1] += self.velocity[1]
			self.position[2] += self.velocity[2]
			
			# Check for player
			found = self.LookforPlayer()
			if found == 'short':
				self.tracking_to -= 1
			if self.tracking_to <= 0:
				self.state = 'attack'
				resources.soundfx['alert'].play()
			
			self.CheckDetonate()
			
		elif self.state == 'attack':
			
			self.MovetoWaypoint(self.h_hackerbot.position)
			self.CheckDetonate()
	
	def hit(self,type):
		self.state = 'dead'
		if self.h_hackerbot.hacking_state == 2 and self.h_hackerbot.target == self:
			self.h_hud.delaytask = 'missile_destroyed'
			self.h_hud.delaytaskto = 150
			self.h_hackerbot.hacking_state = 0
			self.h_hackerbot.h_motion.mbdown = 0
			self.h_hackerbot.hack_to = 0
			self.h_hackerbot.h_motion.viewtype = 0
	
	def render(self):
		if self.h_hackerbot.hacking_state == 2 and self.h_hackerbot.target == self:
			return
		if not self.state == 'dead':
			glLoadIdentity()
			glPushMatrix()
		
			glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
			glRotate(-self.yaw*180/pi, 0, 1, 0)
			glRotate(self.pitch*180/pi, 1, 0, 0)
		
			glScale(1.0, 1.0, 1.0)
			glCallList(resources.model_missile.obj.gl_list)
		
			glPopMatrix()

