#!/usr/bin/python

"""
playerbot.py - classes for player control/motion etc.
"""

import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
#from OpenGL.GLUT import *

from gamedirector3d import *
from math import *
import numpy as np

import resources

class MotionControl(object):
    def __init__(self,parent,controlmap):
        self.parent = parent
        self.CM = controlmap
        self.mouse_sensitivity = 0.005
        
        self.u = 0
        self.d = 0
        self.l = 0
        self.r = 0
        
        self.mbdown = 0
        
        self.viewtype = 0
        self.pitch = 0.0
        self.yaw = 0.0
        self.pitch_last = 0.0
        self.yaw_last = 0.0
        self.position = [0.0,0.0,0.0]
        
        """
        self.utest = 0
        self.dtest = 0
        self.ltest = 0
        self.rtest = 0
        self.putest = 0
        self.pdtest = 0
        self.pltest = 0
        self.prtest = 0
        """
    
    def ProcessKeyEvent(self,event):
    	if self.parent.playeralive == False or self.parent.missioncomplete == True:
    		self.viewtype = 0
    		self.mbdown = 0
    		return
        if event.type == KEYDOWN:
			if event.key == self.CM["U"]:
				self.u = 1
			elif event.key == self.CM["D"]:
				self.d = 1
			elif event.key == self.CM["L"]:
				self.l = 1
			elif event.key == self.CM["R"] and not self.h_hackerbot.hacking_state == 1:
				self.r = 1
			elif event.key == self.CM["View"]:
				if self.h_hackerbot.current_asteroid > -1 and self.h_hackerbot.hacking_state == 0:
					self.viewtype = (self.viewtype+1) % 2
					if self.viewtype == 0:
						self.pitch = self.pitch_last
						self.yaw = self.yaw_last
					elif self.viewtype == 1:
						self.pitch_last = self.pitch
						self.yaw_last = self.yaw
						self.position[0] = self.h_hackerbot.position[0]
						self.position[1] = self.h_hackerbot.position[1]
						self.position[2] = self.h_hackerbot.position[2]
			elif event.key == self.CM["Jump"]:
				if self.h_hackerbot.current_asteroid > -1 and self.h_hackerbot.hacking_state == 0:
					self.h_hackerbot.jump()
					self.viewtype = 0
				elif self.h_hackerbot.hacking_state == 2:
					DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
					dpx = DCM[0][0]
					dpy = DCM[1][0]
					dpz = DCM[2][0]
					self.h_projectiles.New(self.h_hackerbot.target.position+[1.0*dpx,1.0*dpy,1.0*dpz],[10.0*dpx,10.0*dpy,10.0*dpz],0,self.h_hackerbot.target)
			elif event.key == K_r and (self.h_hackerbot.hacking_state == 1 or self.h_hackerbot.hacking_state == 2):
				self.h_hackerbot.hacking_state = 0
				self.mbdown = 0
				if self.h_hackerbot.target.type == 'sentry':
					self.h_hackerbot.target.state = 'patrol'
				elif self.h_hackerbot.target.type == 'missile':
					self.h_hackerbot.target.state = 'attack'
				self.h_hackerbot.hack_to = 0
				self.viewtype = 0
			elif event.key == K_d and self.h_hackerbot.hacking_state == 1:
				self.h_hackerbot.hacking_state = 0
				self.mbdown = 0
				if self.h_hackerbot.target.type == 'sentry':
					self.h_hackerbot.target.state = 'disabled'
					self.h_hackerbot.target.disableto = 30*30
					self.parent.hud.delaytask = 'sentry_disabled'
					self.parent.hud.delaytaskto = 150
				elif self.h_hackerbot.target.type == 'missile':
					self.h_hackerbot.target.state = 'dead'
					self.h_explosions.New(self.h_hackerbot.target.position-self.h_hackerbot.target.velocity,0)
					self.parent.hud.delaytask = 'missile_destroyed'
					self.parent.hud.delaytaskto = 150
				self.h_hackerbot.hack_to = 0
				self.viewtype = 0
			elif event.key == K_t and self.h_hackerbot.hacking_state == 1:
				self.h_hackerbot.hacking_state = 2
				self.mbdown = 0
				self.h_hackerbot.control_to = 30*30
				self.h_hackerbot.hack_to = 0
				self.viewtype = 2
				self.pitch = self.h_hackerbot.target.pitch
				self.yaw = self.h_hackerbot.target.yaw
				self.position[0] = self.h_hackerbot.target.position[0]
				self.position[1] = self.h_hackerbot.target.position[1]
				self.position[2] = self.h_hackerbot.target.position[2]
				
			"""
			elif event.key == K_q:
				DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
				dpx = DCM[0][0]
				dpy = DCM[1][0]
				dpz = DCM[2][0]
				self.h_projectiles.New(self.h_hackerbot.position+[1.0*dpx,1.0*dpy,1.0*dpz],[10.0*dpx,10.0*dpy,10.0*dpz],0,self.h_hackerbot)
			"""
        elif event.type == KEYUP:
            if event.key == self.CM["U"]:
                self.u = 0
            elif event.key == self.CM["D"]:
                self.d = 0
            elif event.key == self.CM["L"]:
                self.l = 0
            elif event.key == self.CM["R"]:
                self.r = 0
        elif event.type == MOUSEBUTTONDOWN:
        	if self.h_hackerbot.hacking_state == 1:
        		pass
        	elif self.viewtype == 1 and not self.h_hackerbot.found_target == False:
        		self.mbdown = 1
        	elif self.h_hackerbot.hacking_state == 2 and self.h_hackerbot.target.type == 'sentry':
				DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
				dpx = DCM[0][0]
				dpy = DCM[1][0]
				dpz = DCM[2][0]
				self.h_projectiles.New(self.h_hackerbot.target.position+[1.0*dpx,1.0*dpy,1.0*dpz],[10.0*dpx,10.0*dpy,10.0*dpz],0,self.h_hackerbot.target)
        elif event.type == MOUSEBUTTONUP:
        	self.mbdown = 0
        
    def UpdateMotion(self):
        rel = [self.parent.mp[0]-320,self.parent.mp[1]-240]
        if self.viewtype == 0: # camera focused on player
			self.pitch -= self.mouse_sensitivity*rel[1]
			if self.pitch >= pi/2:
				self.pitch = pi/2-0.0001
			elif self.pitch <= -pi/2:
				self.pitch = -pi/2+0.0001
			self.yaw += self.mouse_sensitivity*rel[0]
        elif self.viewtype == 1 or self.viewtype == 2: # camera outwards looking/free flying
			self.pitch -= self.mouse_sensitivity*rel[1]
			if self.pitch >= pi/2:
				self.pitch = pi/2-0.0001
			elif self.pitch <= -pi/2:
				self.pitch = -pi/2+0.0001
			self.yaw += self.mouse_sensitivity*rel[0]
			if self.viewtype == 1:
				self.position[0] = self.h_hackerbot.position[0]
				self.position[1] = self.h_hackerbot.position[1]
				self.position[2] = self.h_hackerbot.position[2]
    
    def set_view(self):
    	if self.viewtype == 0: # camera focused on player
    		vx_rot = cos(self.yaw+pi/2)
    		vy_rot = sin(self.yaw+pi/2)
    		glRotate(self.yaw*180/pi, 0, 1, 0)
    		glRotate(-self.pitch*180/pi, vy_rot, 0, -vx_rot)
    		#glTranslate(-self.h_hackerbot.position[1],self.h_hackerbot.position[2],self.h_hackerbot.position[0])
    		DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
    		dist = 20.0
    		dpx = -DCM[0][0]*dist
    		dpy = -DCM[1][0]*dist
    		dpz = -DCM[2][0]*dist
    		if self.h_hackerbot.hacking_state == 0:
    			self.position[0] = self.h_hackerbot.position[0]+dpx
    			self.position[1] = self.h_hackerbot.position[1]+dpy
    			self.position[2] = self.h_hackerbot.position[2]+dpz
    		else:
    			npc = self.h_hackerbot.target
    			self.position[0] = npc.position[0]+dpx
    			self.position[1] = npc.position[1]+dpy
    			self.position[2] = npc.position[2]+dpz
    		#glTranslate(-dpy, dpz, dpx)
    		glTranslate(-self.position[1], self.position[2], self.position[0])
    	elif self.viewtype == 1: # camera outwards looking
    		vx_rot = cos(self.yaw+pi/2)
    		vy_rot = sin(self.yaw+pi/2)
    		glRotate(self.yaw*180/pi, 0, 1, 0)
    		glRotate(-self.pitch*180/pi, vy_rot, 0, -vx_rot)
    		glTranslate(-self.position[1], self.position[2], self.position[0])
    	elif self.viewtype == 2: # camera free flying
			vx_rot = cos(self.yaw+pi/2)
			vy_rot = sin(self.yaw+pi/2)
			"""
			DCM = np.array([[cos(self.pitch)*cos(self.yaw),-sin(self.yaw),sin(self.pitch)*cos(self.yaw)],[cos(self.pitch)*sin(self.yaw),cos(self.yaw),sin(self.pitch)*sin(self.yaw)],[-sin(self.pitch),0.0,cos(self.pitch)]])
			if self.h_hackerbot.target.type == 'missile':
				dpx_b = 1.5
				dpy_b = 0.0
			else:
				dpx_b = 1.0*(self.u-self.d)
				dpy_b = 1.0*(self.r-self.l)
			dpx = DCM[0][0]*dpx_b + DCM[0][1]*dpy_b
			dpy = DCM[1][0]*dpx_b + DCM[1][1]*dpy_b
			dpz = DCM[2][0]*dpx_b + DCM[2][1]*dpy_b
			self.position[0] += dpx
			self.position[1] += dpy
			self.position[2] += dpz
			
			self.h_hackerbot.target.position[0] = self.position[0]
			self.h_hackerbot.target.position[1] = self.position[1]
			self.h_hackerbot.target.position[2] = self.position[2]
			self.h_hackerbot.target.pitch = self.pitch
			self.h_hackerbot.target.yaw = self.yaw
			
			if self.h_hackerbot.target.type == 'sentry':
				self.h_hackerbot.target.CorrectTrajforObstacles(control='player')
				self.position[0] = self.h_hackerbot.target.position[0]
				self.position[1] = self.h_hackerbot.target.position[1]
				self.position[2] = self.h_hackerbot.target.position[2]
			else:
				self.h_hackerbot.target.velocity[0] = dpx
				self.h_hackerbot.target.velocity[1] = dpy
				self.h_hackerbot.target.velocity[2] = dpz
				self.h_hackerbot.target.CheckDetonate()
			"""
			glRotate(self.yaw*180/pi, 0, 1, 0)
			glRotate(-self.pitch*180/pi, vy_rot, 0, -vx_rot)
			glTranslate(-self.position[1], self.position[2], self.position[0])
    	
    	#print "player", self.position[0], self.position[1], self.position[2]
    	#print self.yaw*180/pi, self.pitch*180/pi

class Hackerbot(object):
	def __init__(self,parent,pos=[0.0,0.0,0.0],vel=[0.0,0.0,0.0],yaw=0.0):
		self.parent = parent
		self.current_asteroid = -1
		self.position = np.array(pos)
		self.velocity = np.array(vel)
		self.rotation = [0.0, 0.0, 0.0]
		self.cl_ind = 0
		self.radius = 3.0
		self.heading = 0.0
		self.gait = 0
		
		self.parent.motion.yaw = yaw
		
		#self.max_hack_range = 50.0
		self.hack_to = 0
		self.control_to = 0
		
		self.hacking_state = 0
		self.found_target = False
		self.target = None
	
	def jump(self):
		jump_vel = 1.0
		self.current_asteroid = -1
		DCM = np.array([[cos(self.h_motion.pitch)*cos(self.h_motion.yaw),-sin(self.h_motion.yaw),sin(self.h_motion.pitch)*cos(self.h_motion.yaw)],\
			[cos(self.h_motion.pitch)*sin(self.h_motion.yaw),cos(self.h_motion.yaw),sin(self.h_motion.pitch)*sin(self.h_motion.yaw)],\
			[-sin(self.h_motion.pitch),0.0,cos(self.h_motion.pitch)]])
		self.velocity[0] = DCM[0][0]*jump_vel
		self.velocity[1] = DCM[1][0]*jump_vel
		self.velocity[2] = DCM[2][0]*jump_vel
	
	def Update(self):
		
		# check if actually alive
		if self.parent.playeralive == False:
			self.parent.on_alert = False
			return
		
		if self.hacking_state == 0:
			self.parent.on_alert = False
		
		# first check for hacking state
		if self.hacking_state == 1:
			return
		elif self.hacking_state == 2:
			self.gait = 0
			if self.target.type == 'sentry':
				self.control_to -= 1
				if self.control_to <= 0:
					self.hacking_state = 0
					self.h_motion.mbdown = 0
					self.target.state = 'patrol'
					self.hack_to = 0
					self.h_motion.viewtype = 0
					return
			DCM = np.array([[cos(self.h_motion.pitch)*cos(self.h_motion.yaw),-sin(self.h_motion.yaw),sin(self.h_motion.pitch)*cos(self.h_motion.yaw)],[cos(self.h_motion.pitch)*sin(self.h_motion.yaw),cos(self.h_motion.yaw),sin(self.h_motion.pitch)*sin(self.h_motion.yaw)],[-sin(self.h_motion.pitch),0.0,cos(self.h_motion.pitch)]])
			if self.target.type == 'missile':
				dpx_b = 1.5
				dpy_b = 0.0
			else:
				dpx_b = 1.0*(self.h_motion.u-self.h_motion.d)
				dpy_b = 1.0*(self.h_motion.r-self.h_motion.l)
			dpx = DCM[0][0]*dpx_b + DCM[0][1]*dpy_b
			dpy = DCM[1][0]*dpx_b + DCM[1][1]*dpy_b
			dpz = DCM[2][0]*dpx_b + DCM[2][1]*dpy_b
			self.h_motion.position[0] += dpx
			self.h_motion.position[1] += dpy
			self.h_motion.position[2] += dpz
			
			self.target.position[0] = self.h_motion.position[0]
			self.target.position[1] = self.h_motion.position[1]
			self.target.position[2] = self.h_motion.position[2]
			self.target.pitch = self.h_motion.pitch
			self.target.yaw = self.h_motion.yaw
			
			if self.target.type == 'sentry':
				self.target.CorrectTrajforObstacles(control='player')
				self.h_motion.position[0] = self.target.position[0]
				self.h_motion.position[1] = self.target.position[1]
				self.h_motion.position[2] = self.target.position[2]
			else:
				self.target.velocity[0] = dpx
				self.target.velocity[1] = dpy
				self.target.velocity[2] = dpz
				self.target.CheckDetonate()
			
		# jumping
		if self.current_asteroid == -1:
			self.gait = 0
			self.heading = 0.0
			self.h_motion.viewtype = 0
			self.position[0] += self.velocity[0]
			self.position[1] += self.velocity[1]
			self.position[2] += self.velocity[2]
			for i in xrange(len(self.h_asteroids)): # check for contact
				dist = sqrt(np.sum((self.position - self.h_asteroids[i].position)**2))
				if dist < self.h_asteroids[i].radius:
					self.current_asteroid = i
					break
			if sqrt(np.sum((self.position)**2)) > self.parent.map_limit:
				self.parent.playeralive = False
				self.parent.reasonfail = 'death_float'
				self.parent.fadeto = 90
				self.parent.motion.mbdown = 0
				self.parent.motion.viewtype = 0
				self.hacking_state = 0
		if self.current_asteroid == -1:
			return
			
		asteroid = self.h_asteroids[self.current_asteroid]
		dists = np.sum((self.position - (asteroid.centers+asteroid.position))**2,axis=1)
		averti = np.argmin(dists)
		self.cl_ind = averti
		
		if self.h_motion.viewtype == 0: # on-asteroid motion
			# Get player pointing direction
			DCM = np.array([[cos(self.h_motion.pitch)*cos(self.h_motion.yaw),-sin(self.h_motion.yaw),sin(self.h_motion.pitch)*cos(self.h_motion.yaw)],\
				[cos(self.h_motion.pitch)*sin(self.h_motion.yaw),cos(self.h_motion.yaw),sin(self.h_motion.pitch)*sin(self.h_motion.yaw)],\
				[-sin(self.h_motion.pitch),0.0,cos(self.h_motion.pitch)]])
			
			dpz_b = -0.5*(self.h_motion.u-self.h_motion.d)
			dpy_b = 0.5*(self.h_motion.r-self.h_motion.l)
			dpx = DCM[0][1]*dpy_b + DCM[0][2]*dpz_b
			dpy = DCM[1][1]*dpy_b + DCM[1][2]*dpz_b
			dpz = DCM[2][1]*dpy_b + DCM[2][2]*dpz_b
			
			self.position[0] += dpx
			self.position[1] += dpy
			self.position[2] += dpz
			
			if self.current_asteroid >= 0:
				
				if not (dpz_b == 0 and dpy_b == 0):
					self.gait += 1
				else:
					self.gait = 0
				
				rel_pos = self.h_asteroids[self.current_asteroid].position-self.position
				hordist = sqrt(rel_pos[0]*rel_pos[0] + rel_pos[1]*rel_pos[1])
				if hordist > 0.0:
					drawyaw = atan2(rel_pos[1],rel_pos[0])
					drawpitch = -atan(rel_pos[2]/hordist)
				else:
					drawyaw = 0.0
					if rel_pos[2] > 0.0:
						drawpitch = -pi/2
					else:
						drawpitch = pi/2
				DCM2 = np.array([[cos(drawpitch)*cos(drawyaw),-sin(drawyaw),sin(drawpitch)*cos(drawyaw)],\
					[cos(drawpitch)*sin(drawyaw),cos(drawyaw),sin(drawpitch)*sin(drawyaw)],\
					[-sin(drawpitch),0.0,cos(drawpitch)]])
				
				dpx_a = DCM2[0][0]*dpx + DCM2[1][0]*dpy + DCM2[2][0]*dpz
				dpy_a = DCM2[0][1]*dpx + DCM2[1][1]*dpy + DCM2[2][1]*dpz
				dpz_a = DCM2[0][2]*dpx + DCM2[1][2]*dpy + DCM2[2][2]*dpz
				if not (dpz_a == 0 and dpy_a == 0):
					target_ang = atan2(dpy_a,-dpz_a)
					dang = target_ang - self.heading
					if dang > pi:
						dang = -(2*pi-dang)
					elif dang <= -pi:
						dang = -(2*pi+dang)
					turnrate = 0.2
					if (dang < 0):
						self.heading -= min(turnrate,fabs(dang))
					elif (dang > 0):
						self.heading += min(turnrate,fabs(dang))
					self.heading = (self.heading + pi) % (2*pi) - pi
			
			# enforce constraint to asteroid floor
			radius = sqrt(asteroid.radii[averti])+1.0
			relpos = self.position - np.array(asteroid.position)
			unitrelpos = relpos/sqrt(pow(relpos[0],2)+pow(relpos[1],2)+pow(relpos[2],2))
			newpos = asteroid.position + radius*unitrelpos
			self.position[0] = newpos[0]
			self.position[1] = newpos[1]
			self.position[2] = newpos[2]
			
		elif self.h_motion.viewtype == 1: # looking around
			
			"""
			DCM = np.array([[cos(self.h_motion.pitch)*cos(self.h_motion.yaw),-sin(self.h_motion.yaw),sin(self.h_motion.pitch)*cos(self.h_motion.yaw)],\
				[cos(self.h_motion.pitch)*sin(self.h_motion.yaw),cos(self.h_motion.yaw),sin(self.h_motion.pitch)*sin(self.h_motion.yaw)],\
				[-sin(self.h_motion.pitch),0.0,cos(self.h_motion.pitch)]])
			dpx_b = 1.0*(self.h_motion.u-self.h_motion.d)
			dpy_b = 1.0*(self.h_motion.r-self.h_motion.l)
			dpx = DCM[0][0]*dpx_b + DCM[0][1]*dpy_b
			dpy = DCM[1][0]*dpx_b + DCM[1][1]*dpy_b
			dpz = DCM[2][0]*dpx_b + DCM[2][1]*dpy_b
			self.position[0] += dpx
			self.position[1] += dpy
			self.position[2] += dpz
			"""
			
			# Control HUD stuff
			
			# Check for targets
			DCM = np.array([[cos(self.h_motion.pitch)*cos(self.h_motion.yaw),-sin(self.h_motion.yaw),sin(self.h_motion.pitch)*cos(self.h_motion.yaw)],\
				[cos(self.h_motion.pitch)*sin(self.h_motion.yaw),cos(self.h_motion.yaw),sin(self.h_motion.pitch)*sin(self.h_motion.yaw)],\
				[-sin(self.h_motion.pitch),0.0,cos(self.h_motion.pitch)]])
			
			self.found_target = False
			found_outofrange = False
			for npc in self.parent.npcs:
				relpos = npc.position - self.position
				dpx = DCM[0][0]*relpos[0] + DCM[1][0]*relpos[1] + DCM[2][0]*relpos[2]
				dpy = DCM[0][1]*relpos[0] + DCM[1][1]*relpos[1] + DCM[2][1]*relpos[2]
				dpz = DCM[0][2]*relpos[0] + DCM[1][2]*relpos[1] + DCM[2][2]*relpos[2]
				if dpx > 0:
					ang1 = fabs(dpy/dpx)
					ang2 = fabs(dpz/dpx)
				else:
					ang1 = 1.0
					ang2 = 1.0
				if ang1 < 0.1 and ang2 < 0.1:
					if npc.state == 'disabled':
						name = npc.type+'dis'
						(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs[name].w/2, self.h_hud.window_size[1]/2 + 48)
						self.h_hud.tasks.append((resources.HUD_surfs[name], x, y, 1.0, 0))
						if npc.type == 'sentry':
							time_rev = int(npc.disableto/30)
							(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs['revive'][time_rev].w/2, self.h_hud.window_size[1]/2 + 16)
							self.h_hud.tasks.append((resources.HUD_surfs['revive'][time_rev], x, y, 1.0, 0))
						continue
					else:
						name = npc.type
					if self.hack_to == 0:
						(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs[name].w/2, self.h_hud.window_size[1]/2 + 48)
						self.h_hud.tasks.append((resources.HUD_surfs[name], x, y, 1.0, 0))
					
					# Check not looking through asteroid (too much)
					x0 = astpos = self.h_asteroids[self.current_asteroid].position
					x1 = self.position
					x2 = npc.position
					d1 = sqrt(np.sum((relpos)**2))
					if d1 == 0.0:
						dline = 0.0
					else:
						dline = sqrt(np.sum((np.cross(relpos,(x1-x0)))**2))/d1
					
					if not (dpx > 1.0 and dpx < npc.max_hack_range and dline > (self.h_asteroids[self.current_asteroid].radius-2.0)):
						(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs['hackrange'].w/2, self.h_hud.window_size[1]/2 + 16)
						self.h_hud.tasks.append((resources.HUD_surfs['hackrange'], x, y, 1.0, 0))
						(x, y) = (self.h_hud.window_size[0]/2 - 16, self.h_hud.window_size[1]/2 - 16)
						self.h_hud.tasks.append((resources.hud_target_sprites, x, y, 1.0, 0))
						found_outofrange = True
						self.hack_to = 0
						resources.soundfx['hack'].stop()
					else:
						(x, y) = (self.h_hud.window_size[0]/2 - 16, self.h_hud.window_size[1]/2 - 16)
						self.h_hud.tasks.append((resources.hud_target_sprites, x, y, 1.0, 2))
						if self.hack_to == 0:
							(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs['tohack'].w/2, self.h_hud.window_size[1]/2 + 16)
							self.h_hud.tasks.append((resources.HUD_surfs['tohack'], x, y, 1.0, 0))
						else:
							if npc.type == 'datavault':
								(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs['hackingdata'].w/2, self.h_hud.window_size[1]/2 + 48)
								self.h_hud.tasks.append((resources.HUD_surfs['hackingdata'], x, y, 1.0, 0))
								self.h_hud.render_hack_progress(self.hack_to,npc.hack_to)
							else:
								(x, y) = (self.h_hud.window_size[0]/2 - resources.HUD_surfs['hacking'].w/2, self.h_hud.window_size[1]/2 + 48)
								self.h_hud.tasks.append((resources.HUD_surfs['hacking'], x, y, 1.0, 0))
								self.h_hud.render_hack_progress(self.hack_to,npc.hack_to)
						self.found_target = npc
						if self.h_motion.mbdown:
							if self.hack_to == 0:
								resources.soundfx['hack'].play()
							self.hack_to += 1
							if self.hack_to == npc.hack_to:
								self.h_motion.mbdown = False
								resources.soundfx['hack'].stop()
								resources.soundfx['success'].play()
								if npc.type == 'datavault':
									npc.state = 'disabled'
									self.parent.datavaultleft -= 1
									if self.parent.datavaultleft == 0:
										self.parent.fadeto = 150
										self.parent.missioncomplete = True
										self.h_motion.viewtype = 0
									else:
										self.h_hud.delaytask = 'datavault'
										self.h_hud.delaytaskto = 150
								elif npc.type == 'shieldgen':
									npc.state = 'disabled'
									npc.h_obj.active = False
									self.h_hud.delaytask = 'shield'
									self.h_hud.delaytaskto = 150
								elif npc.type == 'turret':
									npc.state = 'disabled'
									self.h_hud.delaytask = 'turret_disabled'
									self.h_hud.delaytaskto = 150
								elif npc.type == 'sentry':
									self.parent.console.new('access')
									self.hacking_state = 1
									self.target = npc
									self.target.state = 'hacked'
									self.hack_to = 0
									self.h_motion.viewtype = 0
								elif npc.type == 'missile':
									self.parent.console.new('access')
									self.hacking_state = 1
									self.target = npc
									self.target.state = 'hacked'
									self.hack_to = 0
									self.h_motion.viewtype = 0
								break
						else:
							self.hack_to = 0
							resources.soundfx['hack'].stop()
			if self.found_target == False and found_outofrange == False:
				(x, y) = (self.h_hud.window_size[0]/2 - 16, self.h_hud.window_size[1]/2 - 16)
				self.h_hud.tasks.append((resources.hud_target_sprites, x, y, 1.0, 1))
				self.hack_to = 0
				resources.soundfx['hack'].stop()
			
	
	def render(self):
		if self.h_motion.viewtype == 1 or self.parent.playeralive == False:
			return
		
		if self.current_asteroid == -1:
			rel_pos = self.velocity
			drawheading = 0.0
		else:
			rel_pos = self.h_asteroids[self.current_asteroid].position-self.position
			drawheading = self.heading
		
		hordist = sqrt(rel_pos[0]*rel_pos[0] + rel_pos[1]*rel_pos[1])
		if hordist > 0.0:
			drawyaw = atan2(rel_pos[1],rel_pos[0])
			drawpitch = -atan(rel_pos[2]/hordist)
		else:
			drawyaw = 0.0
			if rel_pos[2] > 0.0:
				drawpitch = -pi/2
			else:
				drawpitch = pi/2
		
		if self.current_asteroid == -1:
			drawpitch -= pi/2
		
		glLoadIdentity()
		glPushMatrix()
		
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(-drawyaw*180/pi, 0, 1, 0)
		glRotate(drawpitch*180/pi, 1, 0, 0)
		glRotate(-drawheading*180/pi, 0, 0, 1)
		
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbot.obj.gl_list)
		
		glPopMatrix()
		
		# draw legs
		walkmag = 30
		walkperiod = 0.5
		if self.gait > 0:
			#print walkperiod*self.gait
			ang1 = walkmag*sin(walkperiod*self.gait)
			ang2 = walkmag*sin(walkperiod*self.gait)
			ang3 = 10*sin(walkperiod*self.gait+pi/2)
			ang4 = -10*sin(walkperiod*self.gait)
		else:
			ang1 = 0
			ang2 = 0
			ang3 = 0
			ang4 = 0
		
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(-drawyaw*180/pi, 0, 1, 0)
		glRotate(drawpitch*180/pi, 1, 0, 0)
		glRotate(-drawheading*180/pi, 0, 0, 1)
		glTranslate(*(-0.7,-(-0.3),-(0.4)))
		if self.current_asteroid == -1:
			glRotate(-90, 1, 0, 0)#60
			glRotate(90, 0, 0, 1)#60
		else:
			glRotate(60+ang1, 0, 0, 1)#60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegfront.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(-drawyaw*180/pi, 0, 1, 0)
		glRotate(drawpitch*180/pi, 1, 0, 0)
		glRotate(-drawheading*180/pi, 0, 0, 1)
		glTranslate(*(0.7,-(-0.3),-(0.4)))
		if self.current_asteroid == -1:
			glRotate(-90, 1, 0, 0)#60
			glRotate(-90, 0, 0, 1)#60
		else:
			glRotate(-60+ang2, 0, 0, 1)#-60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegfront.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(-drawyaw*180/pi, 0, 1, 0)
		glRotate(drawpitch*180/pi, 1, 0, 0)
		glRotate(-drawheading*180/pi, 0, 0, 1)
		glTranslate(*(-0.7,-(0.5),-(0.4)))
		if self.current_asteroid == -1:
			glRotate(-90, 1, 0, 0)#60
		else:
			glRotate(160+ang3, 0, 0, 1)#60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegback.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(self.position[1],-self.position[2],-self.position[0]))
		glRotate(-drawyaw*180/pi, 0, 1, 0)
		glRotate(drawpitch*180/pi, 1, 0, 0)
		glRotate(-drawheading*180/pi, 0, 0, 1)
		glTranslate(*(0.7,-(0.5),-(0.4)))
		if self.current_asteroid == -1:
			glRotate(-90, 1, 0, 0)#60
		else:
			glRotate(-160+ang4, 0, 0, 1)#-60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegback.obj.gl_list)
		glPopMatrix()
		

