#!/usr/bin/python

"""
titlescenes.py - scene classes for titlescreen and victory scene
"""

import os
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
#from OpenGL.GLUT import *

from gamedirector3d import *
from math import *
import numpy as np

import resources

class Coords(object):
	pass

class TitleScene(GameScene):
    def __init__(self, director, window_size):
        super(TitleScene, self).__init__(director)
        self.window_size = window_size
        
        self.sprite_heights = {}
        self.sprite_heights['logo'] = 150
        self.sprite_heights['new_game'] = -100
        self.sprite_heights['continue'] = -150
    
    def on_switchto(self, switchtoargs):
        
		# Initialise openGL stuff
		glLightfv(GL_LIGHT0, GL_POSITION,  (100, 200, 100, 0.0))
		glLightfv(GL_LIGHT0, GL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
		glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.5, 0.5, 0.5, 1.0))
		glEnable(GL_LIGHT0)
		glEnable(GL_LIGHTING)
		glEnable(GL_COLOR_MATERIAL)
		glEnable(GL_DEPTH_TEST)
		glShadeModel(GL_SMOOTH)
		
		# Initialise motion controls
		self.mp = pygame.mouse.get_pos()
		pygame.mouse.set_visible(True)
		
		# start music
		pygame.mixer.music.stop()
		pygame.mixer.music.load(resources.musicpaths['title'])
		pygame.mixer.music.set_volume(0.5)
		pygame.mixer.music.play(-1)
		
		self.fadeto = 30
		self.starting = False
		self.type = 'new_game'
		self.can_continue = False
		self.select_type = None
		
		# check if save game available
		if os.path.isfile(self.h_progress_data.savepath):
			self.can_continue = True
		
		self.hackerbot_angle = 0.0
		
		# reset maingame music
		self.h_maingame.current_music = 'none'

    def check_buttons(self):
    	
    	coords_new = Coords()
    	coords_new.x = self.window_size[0]/2 - resources.title_sprites['new_game_off'].w/2
    	coords_new.y = self.window_size[1]/2 - self.sprite_heights['new_game']
    	coords_new.w = resources.title_sprites['new_game_off'].w
    	coords_new.h = resources.title_sprites['new_game_off'].h
    	
    	coords_continue = Coords()
    	coords_continue.x = self.window_size[0]/2 - resources.title_sprites['continue_off'].w/2
    	coords_continue.y = self.window_size[1]/2 - self.sprite_heights['continue']
    	coords_continue.w = resources.title_sprites['continue_off'].w
    	coords_continue.h = resources.title_sprites['continue_off'].h
    	
    	if self.mp[0] >= coords_new.x and self.mp[0] <= (coords_new.x+coords_new.w) and self.mp[1] >= coords_new.y and self.mp[1] <= (coords_new.y+coords_new.h):
    		self.select_type = 'new_game'
    	elif self.can_continue and self.mp[0] >= coords_continue.x and self.mp[0] <= (coords_continue.x+coords_continue.w) and self.mp[1] >= coords_continue.y and self.mp[1] <= (coords_continue.y+coords_continue.h):
    		self.select_type = 'continue'
    	else:
    		self.select_type = None

    def on_update(self):
        
        # update music volume based on fade out
        if self.starting == True:
        	pygame.mixer.music.set_volume((0.5/30.0)*self.fadeto)
        
        # check buttons
        self.check_buttons()
        
        # Update timeouts
        if self.fadeto > 0:
        	self.fadeto -= 1
        if self.starting == True and self.fadeto == 1:
        	if self.type == 'new_game':
        		self.director.change_scene('maingame', [resources.MissionData['levels'][0]])
        	elif self.type == 'continue':
        		self.h_progress_data.LoadProgressData()
                self.director.change_scene('maingame', [self.h_progress_data.current_level])
        
        # Update spinning
        self.hackerbot_angle += 5.0
    
    def on_event(self, events):
    	self.mp = pygame.mouse.get_pos()
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                self.director.change_scene(None, [])
            if event.type == MOUSEBUTTONDOWN and not self.starting:
            	if not self.select_type == None:
            		self.starting = True
            		self.fadeto = 30
            		self.type = self.select_type
    
    def render_fade(self, screen):
		if not self.starting:
			fade = self.fadeto/30.0
		else:
			fade = max(0.0,1.0 - self.fadeto/30.0)
		glBegin(GL_QUADS)
		glColor(0, 0, 0, fade)
		glVertex2f(0, 0)
		glVertex2f(self.window_size[0], 0)
		glVertex2f(self.window_size[0], self.window_size[1])
		glVertex2f(0, self.window_size[1])
		glEnd()
    
    def draw(self, screen):
    	
    	# draw background
		glMatrixMode(GL_PROJECTION)
		
		glClearColor(*(0.0,0.0,0.0,0.0))
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		
		glLoadIdentity()
		glViewport(0, 0, self.window_size[0], self.window_size[1])
		glOrtho(0, self.window_size[0], 0, self.window_size[1], -50, 50)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		glDisable(GL_LIGHTING)
		glDisable(GL_DEPTH_TEST)

		glPushMatrix()
		sprite = resources.console_text_sprites['access']
		resources.console_text_sprites['access2'].render(self.window_size[0]/2-sprite.size/2,-200,0.75,0)
		glPopMatrix()
    	
		# Initialise view
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(60.0, self.window_size[0]/float(self.window_size[1]), 1, 1000.0)
		
		# setup camera perspective on the world
		glTranslate(0.0, 1.0, -10.0)
		glRotate(30.0, 1, 0, 0)
		
		# clear screen
		glMatrixMode(GL_MODELVIEW)
		#glClearColor(*(0.0,0.0,0.0,0.0))
		#glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		
		glDisable(GL_LIGHTING)
		glDisable(GL_DEPTH_TEST)
		glDepthMask(GL_FALSE)
		glEnable(GL_LIGHTING)
		glEnable(GL_DEPTH_TEST)
		glDepthMask(GL_TRUE)
		
		# render spinning hackerbot
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(0.0,0.0,0.0))
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbot.obj.gl_list)
		glPopMatrix()
        
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(-0.7,-(-0.3),-(0.4)))
		glRotate(60, 0, 0, 1)#60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegfront.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(0.7,-(-0.3),-(0.4)))
		glRotate(-60, 0, 0, 1)#-60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegfront.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(-0.7,-(0.5),-(0.4)))
		glRotate(160, 0, 0, 1)#60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegback.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(0.7,-(0.5),-(0.4)))
		glRotate(-160, 0, 0, 1)#-60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegback.obj.gl_list)
		glPopMatrix()
        
		# draw buttons
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		glViewport(0, 0, self.window_size[0], self.window_size[1])
		glOrtho(0, self.window_size[0], 0, self.window_size[1], -50, 50)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		glDisable(GL_LIGHTING)
		glDisable(GL_DEPTH_TEST)

		glPushMatrix()
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		x = self.window_size[0]/2 - resources.title_sprites['logo'].w/2
		y = self.window_size[1]/2 + self.sprite_heights['logo']
		resources.title_sprites['logo'].render(x,y,1.0,0.0)

		x = self.window_size[0]/2 - resources.title_sprites['new_game_off'].w/2
		y = self.window_size[1]/2 + self.sprite_heights['new_game'] - resources.title_sprites['new_game_off'].h
		if self.select_type == 'new_game':
			resources.title_sprites['new_game_on'].render(x,y,1.0,0.0)
		else:
			resources.title_sprites['new_game_off'].render(x,y,1.0,0.0)
		
		if self.can_continue:
			x = self.window_size[0]/2 - resources.title_sprites['continue_off'].w/2
			y = self.window_size[1]/2 + self.sprite_heights['continue'] - resources.title_sprites['continue_off'].h
			if self.select_type == 'continue':
				resources.title_sprites['continue_on'].render(x,y,1.0,0.0)
			else:
				resources.title_sprites['continue_off'].render(x,y,1.0,0.0)
		
		if self.fadeto > 0:
			self.render_fade(screen)
        	
		glDisable(GL_BLEND)
		glPopMatrix()
    
    def on_draw(self, screen):
        self.draw(screen)

class VictoryScene(GameScene):
    def __init__(self, director, window_size):
        super(VictoryScene, self).__init__(director)
        self.window_size = window_size
    	
    	self.sprite_heights = {}
        self.sprite_heights['victory'] = 150
        self.sprite_heights['victory1'] = -150
        self.sprite_heights['victory2'] = -180
        self.sprite_heights['victory3'] = -210
    	
    def on_switchto(self, switchtoargs):
        
		# Initialise openGL stuff
		glLightfv(GL_LIGHT0, GL_POSITION,  (100, 200, 100, 0.0))
		glLightfv(GL_LIGHT0, GL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
		glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.5, 0.5, 0.5, 1.0))
		glEnable(GL_LIGHT0)
		glEnable(GL_LIGHTING)
		glEnable(GL_COLOR_MATERIAL)
		glEnable(GL_DEPTH_TEST)
		glShadeModel(GL_SMOOTH)
		
		# Initialise motion controls
		self.mp = pygame.mouse.get_pos()
		pygame.mouse.set_visible(True)
		
		# start music
		pygame.mixer.music.stop()
		pygame.mixer.music.load(resources.musicpaths['deceptive'])
		pygame.mixer.music.set_volume(0.5)
		pygame.mixer.music.play(-1)
		
		self.fadeto = 30
		self.starting = False
		
		self.hackerbot_angle = 0.0

    def on_update(self):
        
        # update music volume based on fade out
        if self.starting == True:
        	pygame.mixer.music.set_volume((0.5/30.0)*self.fadeto)
        
        # Update timeouts
        if self.fadeto > 0:
        	self.fadeto -= 1
        if self.starting == True and self.fadeto == 1:
        	self.director.change_scene('titlescene', [])
        
        # Update spinning
        self.hackerbot_angle += 5.0
    
    def on_event(self, events):
    	self.mp = pygame.mouse.get_pos()
        for event in events:
            if event.type == KEYDOWN and not self.starting:
                self.starting = True
                self.fadeto = 30
            if event.type == MOUSEBUTTONDOWN and not self.starting:
            	self.starting = True
            	self.fadeto = 30
    
    def render_fade(self, screen):
		if not self.starting:
			fade = self.fadeto/30.0
		else:
			fade = max(0.0,1.0 - self.fadeto/30.0)
		glBegin(GL_QUADS)
		glColor(0, 0, 0, fade)
		glVertex2f(0, 0)
		glVertex2f(self.window_size[0], 0)
		glVertex2f(self.window_size[0], self.window_size[1])
		glVertex2f(0, self.window_size[1])
		glEnd()
    
    def draw(self, screen):
    	
		# Initialise view
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(60.0, self.window_size[0]/float(self.window_size[1]), 1, 1000.0)
		
		# setup camera perspective on the world
		glTranslate(0.0, 1.0, -10.0)
		glRotate(30.0, 1, 0, 0)
		
		# clear screen
		glMatrixMode(GL_MODELVIEW)
		glClearColor(*(0.0,0.0,0.0,0.0))
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		
		glDisable(GL_LIGHTING)
		glDisable(GL_DEPTH_TEST)
		glDepthMask(GL_FALSE)
		glEnable(GL_LIGHTING)
		glEnable(GL_DEPTH_TEST)
		glDepthMask(GL_TRUE)
		
		# render spinning hackerbot
		glLoadIdentity()
		glPushMatrix()
		glTranslate(*(0.0,0.0,0.0))
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbot.obj.gl_list)
		glPopMatrix()
        
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(-0.7,-(-0.3),-(0.4)))
		glRotate(60, 0, 0, 1)#60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegfront.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(0.7,-(-0.3),-(0.4)))
		glRotate(-60, 0, 0, 1)#-60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegfront.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(-0.7,-(0.5),-(0.4)))
		glRotate(160, 0, 0, 1)#60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegback.obj.gl_list)
		glPopMatrix()
		
		glLoadIdentity()
		glPushMatrix()
		glRotate(-90, 1, 0, 0)
		glRotate(-self.hackerbot_angle, 0, 0, 1)
		glTranslate(*(0.7,-(0.5),-(0.4)))
		glRotate(-160, 0, 0, 1)#-60
		glScale(1.0, 1.0, 1.0)
		glCallList(resources.model_hackerbotlegback.obj.gl_list)
		glPopMatrix()
        
		# draw buttons
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		glViewport(0, 0, self.window_size[0], self.window_size[1])
		glOrtho(0, self.window_size[0], 0, self.window_size[1], -50, 50)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		glDisable(GL_LIGHTING)
		glDisable(GL_DEPTH_TEST)

		glPushMatrix()
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		x = self.window_size[0]/2 - resources.title_sprites['victory'].w/2
		y = self.window_size[1]/2 + self.sprite_heights['victory']
		resources.title_sprites['victory'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.title_sprites['victory1'].w/2
		y = self.window_size[1]/2 + self.sprite_heights['victory1']
		resources.title_sprites['victory1'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.title_sprites['victory2'].w/2
		y = self.window_size[1]/2 + self.sprite_heights['victory2']
		resources.title_sprites['victory2'].render(x,y,1.0,0.0)
		x = self.window_size[0]/2 - resources.title_sprites['victory3'].w/2
		y = self.window_size[1]/2 + self.sprite_heights['victory3']
		resources.title_sprites['victory3'].render(x,y,1.0,0.0)
		
		if self.fadeto > 0:
			self.render_fade(screen)
        	
		glDisable(GL_BLEND)
		glPopMatrix()
    
    def on_draw(self, screen):
        self.draw(screen)


